@@ -8,11 +8,11 @@
  **
  ** See the file "LICENSE" for more information
  **
+ ** $Id: CatchmentPortrayal.java 685 2012-06-25 03:58:31Z mcoletti $
  **/
 package sim.app.geo.sickStudents;
 
 import java.awt.Graphics2D;
-
 import sim.portrayal.DrawInfo2D;
 import sim.portrayal.geo.GeomPortrayal;
 import sim.util.geo.MasonGeometry;
@@ -23,6 +23,8 @@ public class CatchmentPortrayal extends GeomPortrayal
 	private static final long serialVersionUID = 6026649920581400781L;
 
 	SimpleColorMap colorMap = null;
+
+    // FIXME: this should be using Mason's RNG
 	public double proportionSick = Math.random();
 	SickStudentsModel model;
 	
@@ -33,13 +35,20 @@ public class CatchmentPortrayal extends GeomPortrayal
 		this.model = model;
 	}
 	
+    @Override
     public void draw(Object object, Graphics2D graphics, DrawInfo2D info)
     {
     	MasonGeometry mg = (MasonGeometry)object;
-    	Integer num = (Integer)SickStudentsModel.getAttribute(mg, "SCHOOL_NUM");
+    	Integer num = mg.getIntegerAttribute("SCHOOL_NUM");
     	School s = model.schoolMap.get(num);
-    	proportionSick = s.getProportionOfSickStudents();
-    	paint = colorMap.getColor(proportionSick);
+
+        // FIXME: why is this occasionally null?
+        if (s != null)
+        {
+            proportionSick = s.getProportionOfSickStudents();
+            paint = colorMap.getColor(proportionSick);
+        }
+
         super.draw(object, graphics, info);    
     }
 }
